#!/usr/bin/env python3

# procedure to get start and end times from Level-0 fpacket files for SPEXone
#  The times are extracted from the first fpacket of each image fpacket group.
#  Reference: SPEXone TMTC Database, SPX1-TN-004

__version__ = '1.0.0_2022-08-19'

from l0info_utils import read_packet, read_apid, get_anc_packet_time

def l0info_spex(args, fh, output):
        
    status = 0
    if args.verbose:
        print("Reading HARP2 science data.")

    # Skip CFE header
    fh.seek(64)
    bSPW = True

    # Find first science fpacket
    apid = 0
    packetLength = 1
    firstp = 0
    while ((apid != 848) or (not firstp)) and (packetLength > 0):
        fpacket, packetLength = read_packet(fh, bSPW)
        apid = read_apid(fpacket)
        if fpacket: firstp = (fpacket[2] & 64)>>6
        
    # If no science packets, rewind and get times from HKT packets
    if (packetLength==0):
        print("No science packets found in file.")
        status = 110
        fh.seek(64)
        apid = 0
        packetLength = 1
        while (apid != 800) and (packetLength > 0):
            fpacket, packetLength = read_packet(fh, bSPW)
            apid = read_apid(fpacket)

        if fpacket:
            mpacket = fpacket
            try:
                stime = get_anc_packet_time(mpacket[6:12],0,'msec')
                print("start_time=%s" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
                if output:
                    output.write("start_time=%s\n" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
            except:
                return 120
                
            while packetLength > 0:
                fpacket, packetLength = read_packet(fh, bSPW)
                apid = read_apid(fpacket)
                if (apid == 800) and fpacket: mpacket = fpacket
                
            etime = get_anc_packet_time(mpacket[6:12],0,'msec')
    else:
        # Get start time
        try:
            stime = get_anc_packet_time(fpacket[300:306],0,'msec')
            print("start_time=%s" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
            if output:
                output.write("start_time=%s\n" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
        except:
            return 120

        # Read to end of file
        epacket = fpacket
        while packetLength > 0:
            apid = 0
            firstp = 0
            while ((apid != 848) or (not firstp)) and (packetLength > 0):
                fpacket, packetLength = read_packet(fh, bSPW)
                apid = read_apid(fpacket)
                if fpacket: firstp = (fpacket[2] & 64)>>6
                
            if fpacket: epacket = fpacket
        
        etime = get_anc_packet_time(epacket[300:306],0,'msec')

    print("stop_time=%s" % etime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
    if output:
        output.write("stop_time=%s\n" % etime.strftime('%Y-%m-%dT%H:%M:%S.%f'))

    return status
